cbm = {}

cbm.path = {
    gfx = "gfx/circular-buy-menu/"
}

cbm.player = {}

---------------------
--      CONFIG     --
---------------------

cbm[1] = { -- Terrorists Menu
    {
        title = "Pistol",
        content = {{1, 500}, {2, 400}, {3, 650}, {4, 600}, {5, 1000}}
    },
    {
        title = "Shotgun",
        content = {{10, 1700}, {11, 3000}}
    },
    {
        title = "Sub Machine Gun",
        content = {{20, 1500}, {23, 1400}, {22, 2350}, {24, 1700}}
    },
    {
        title = "Rifle",
        content = {{38, 2000}, {30, 2500}, {31, 3500}, {34, 2750}, {35, 4750}, {36, 5000}},
    },
    {
        title = "Machine Gun",
        content = {{40, 5750}}
    },
    {61, 50},
    {62, 50},
    {
        title = "Equipment",
        content = {{57, 650}, {58, 1000}, {51, 300}, {52, 200}, {53, 300}, {54, 150}, {59, 1250}}
    }   
}

cbm[2] = { -- Counter-Terrorists Menu
    {
        title = "Pistol",
        content = {{1, 500}, {2, 400}, {3, 650}, {4, 600}, {6, 750}}
    },
    {
        title = "Shotgun",
        content = {{10, 1700}, {11, 3000}}
    },
    {
        title = "Sub Machine Gun",
        content = {{20, 1500}, {21, 1250}, {22, 2350}, {24, 1700}},
    },
    {
        title = "Rifle",
        content = {{39, 2250}, {32, 3100}, {33, 3500}, {34, 2750}, {35, 4750}, {37, 4200}},
    },
    {
        title = "Machine Gun",
        content = {{40, 5750}}
    },
    {61, 50},
    {62, 50},
    {
        title = "Equipment",
        content = {{57, 650}, {58, 1000}, {51, 300}, {52, 200}, {53, 300}, {54, 150}, {56, 200}, {59, 1250}}
    }   
}

cbm.color = {255, 255, 255}

cbm.light = true

-- Do not touch this!
cbm.playerInit = {
    open = false,
    content = {},

    lineImage = {}
}

---------------------
--     SETTINGS    --
---------------------

parse("mp_hudscale 1")
parse("mp_buymenu -")

---------------------
--       LIB       --
---------------------

function io.exists(file)
    local ok, err, code = os.rename(file, file)
    if not ok then
       if code == 13 then
          -- Permission denied, but it exists
          return true
       end
    end
    return ok, err
end

function table.contains(tab, value)
    for _, v in pairs(tab) do
        if (v == value) then
            return true
        end
    end

    return false
end

function lengthdirX(direction, length)
	return math.cos(math.rad(direction - 90)) * length
end

function lengthdirY(direction, length)
	return math.sin(math.rad(direction - 90)) * length
end

function deepcopy(object)
	local lookup_table = {}
	local function _copy(object)
		if type(object) ~= "table" then
			return object
		elseif lookup_table[object] then
			return lookup_table[object]
		end
		local new_table = {}
		lookup_table[object] = new_table
		for index, value in pairs(object) do
			new_table[_copy(index)] = _copy(value)
		end
		return setmetatable(new_table, getmetatable(object))
	end
	return _copy(object)
end

---------------------
--       CORE      --
---------------------

function cbm.errorMsg(id, msg)
    msg2(id, "\169255000000"..msg.."@C")
end

function cbm.hasEnoughMoney(id, money)
    return player(id, "money") >= money
end 

function cbm.isPrimary(weapon)
    local primaryWpns = {10, 11, 20, 21, 22, 23, 24, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 45, 46, 47, 48, 49, 88, 90, 91}

    if table.contains(primaryWpns, weapon) then
        return true
    end

    return false
end

function cbm.isSecondary(weapon)
    local secondaryWpns = {1, 2, 3, 4, 5, 6}

    if table.contains(secondaryWpns, weapon) then
        return true
    end

    return false
end

function cbm.getPrimaryWeapon(id)
    for k, v in pairs(playerweapons(id)) do
        if cbm.isPrimary(v) then
            return {v, playerammo(id, v)}
        end
    end

    return false
end

function cbm.getSecondaryWeapon(id)
    for k, v in pairs(playerweapons(id)) do
        if cbm.isSecondary(v) then
            return {v, playerammo(id, v)}
        end
    end

    return false
end

function cbm.hasItem(id, item)
    if item == 57 then
        if player(id, "armor") >= 65 then
            return true
        end
    elseif item == 58 then
        if player(id, "armor") >= 100 then
            return true
        end
    elseif item == 79 or item == 80 or item == 81 or item == 82 or item == 83 or item == 84 then
        if player(id, "armor") >= 200 + (item - 78) then
            return true
        end
    elseif item == 56 then 
        return player(id, "defusekit")
    elseif item == 59 then
        return player(id, "nightvision")
    elseif item == 60 then
        return player(id, "gasmask")
    end

    for k, v in pairs(playerweapons(id)) do
        if v == item then
            return true
        end
    end

    return false
end

function cbm.hasBuyTimePassed()
    return game("timeleft") >= (game("mp_roundtime") * 60) - (game("mp_buytime") * 60)
end

function cbm.isInBuyZone(id)
    return inentityzone(player(id, "tilex"), player(id, "tiley"), player(id, "team") - 1)
end

function cbm.getItemImagePath(weapon)
    local path = "gfx/weapons/"

    path = path..itemtype(weapon, "name"):lower():gsub("%s+", ""):gsub("-", "")

    if io.exists(path.."_d.bmp") then
        path = path.."_d"
    end

    return path..".bmp"
end

function cbm.drop(id, item, ammoIn, ammo)
    parse("spawnitem "..item.." "..player(id, "tilex").." "..player(id, "tiley").." "..ammoIn.." "..ammo)

    parse('sv_soundpos "items/drop.wav" '..player(id, "x")..' '..player(id, "y"))

    parse("strip "..id.." "..item)
end

function cbm.text(id, textID, text, x, y, align)
    align = align or 1
    parse('hudtxt2 '..id..' '..textID..' "'..text..'" '..x..' '..y..' '..align..' 1')
end

function cbm.drawLine(id, x1, y1, x2, y2)
    local dir = -math.deg(math.atan2(x1 - x2, y1 - y2))
    local dist = math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
    local x, y = x1 + math.sin(math.rad(dir)) * dist/2, y1 - math.cos(math.rad(dir)) * dist/2
    local img = image(cbm.path.gfx.."1x1.png", x, y, 2, id)
    imagepos(img, x, y, dir)
    imagecolor(img, 155, 155, 155)
    imagescale(img, 1, dist)
    return img
end

function cbm.drawPie(id, x, y, rot, angle, length)
    local dir1 = rot - angle / 2
    local dir2 = rot + angle / 2

    return {
        startX = x + lengthdirX(rot, length / 6),
        startY = y + lengthdirY(rot, length / 6),
        centerX = x + lengthdirX(rot, length / 1.5),
        centerY = y + lengthdirY(rot, length / 1.5),
        line1 = cbm.drawLine(id, x, y, x + lengthdirX(dir1, length), y + lengthdirY(dir1, length)),
        line2 = cbm.drawLine(id, x, y, x + lengthdirX(dir2, length), y + lengthdirY(dir2, length))
    }
end

if not cbm.light then
    function cbm.updateSidePanel(id, title, img, texts)
        local ply = cbm.player[id]

        if not ply.open then
            return
        end

        cbm.removeSidePanel(id)

        local y = 74
        cbm.text(id, 129, "\169255255255"..title, 660, y, 0)

        
        if img then
            y = y + 50
            ply.sidePanelImage = image(img, 730, y, 2, id)
            y = y + 50
        else
            y = y + 20
        end

        for i = 1, #texts do
            cbm.text(id, 129 + i, "\169155155155"..texts[i][1], 660, y, 0)
            cbm.text(id, 129 + #texts + i, "\169255255255"..texts[i][2], 800, y, 2)

            y = y + 15
        end
    end


    function cbm.removeSidePanel(id)
        local ply = cbm.player[id]

        if ply.sidePanelImage then
            freeimage(ply.sidePanelImage)
            ply.sidePanelImage = nil
        end

        for i = 129, 147 do
            parse('hudtxt2 '..id..' '..i)
        end
    end
end

function cbm.open(id, content)
    local ply = cbm.player[id]

    if not content or type(content) ~= "table" or #content == 0 then
        erro("Content should be specified as a non-empty table.")
        return
    end

    if #content > 9 then
        error("Buy menu cannot have more than 9 slots.")
        return
    end

    if ply.open then
        cbm.close(id)
    end

    ply.image = image(cbm.path.gfx.."menu"..(cbm.light and "-light" or "")..".png", 425, 240, 2, id)
    imagecolor(ply.image, cbm.color[1], cbm.color[2], cbm.color[3])

    local buttons = ""

    local angle = 360 / #content

    for slot = 1, #content do
        local pie = cbm.drawPie(id, 425, 240, slot * angle, angle, 194)
    
        table.insert(ply.lineImage, pie.line1)
        table.insert(ply.lineImage, pie.line2)

        local buttonName

        cbm.text(id, 110 + slot, "\169155155155"..slot, pie.startX, pie.startY)

        if content[slot].title then -- Check if it is a submenu
            cbm.text(id, 119 + slot, "\169255255255"..content[slot].title, pie.centerX, pie.centerY)

            buttonName = content[slot].title
        else
            local itemTypeID = content[slot][1]
            local itemName = itemtype(itemTypeID, "name")
            local itemPrice = content[slot][2]

            local hasEnoughMoney = cbm.hasEnoughMoney(id, itemPrice)
            local hasItem = cbm.hasItem(id, itemTypeID)

            local mainColor = hasEnoughMoney and (hasItem and "155155155" or "255255255") or "255000000"
            local moneyColor = hasEnoughMoney and "\169000255000" or ""

            cbm.text(id, 119 + slot, "\169"..mainColor..itemName.." "..moneyColor.."$"..itemPrice, pie.centerX, pie.centerY)

            buttonName = itemName
        end

        buttons = buttons..","..buttonName
    end

    menu(id, "Circular Buy Menu@i"..buttons)

    ply.open = true
    ply.content = content
end

function cbm.getHoveredSlot(id)
    local ply = cbm.player[id]

    if not ply.open then
        return
    end

    local content, rot = ply.content, player(id, "rot")

    local angle = 360 / #content

    for slot = 1, #content do
        local range = angle / 2
        local slotDir = slot * angle
        local diff = (rot - slotDir + 180 + 360) % 360 - 180
        if (diff <= range and diff > -range) then
            return slot
        end
    end
    
    return 1
end

function cbm.use(id, slot)
    local ply = cbm.player[id]

    if not ply.open then
        return
    end

    local content = ply.content

    cbm.close(id)

    if content[slot].title then -- Check if it is a submenu
        cbm.open(id, content[slot].content)
    else
        if cbm.hasBuyTimePassed() then
            if cbm.isInBuyZone(id) then
                local typeID = content[slot][1]
                local price = content[slot][2]

                if cbm.hasEnoughMoney(id, price) then
                    if not cbm.hasItem(id, typeID) then
                        if cbm.isSecondary(typeID) then
                            local sec = cbm.getSecondaryWeapon(id)

                            if sec then
                                cbm.drop(id, sec[1], sec[2], sec[3])
                            end
                        elseif cbm.isPrimary(typeID) then
                            local pri = cbm.getPrimaryWeapon(id)

                            if pri then
                                cbm.drop(id, pri[1], pri[2], pri[3])
                            end
                        end

                        if typeID == 56 or typeID == 60 or typeID == 59 then
                            local x, y = player(id, "x"), player(id, "y")

                            parse("spawnitem "..typeID.." "..player(id, "tilex").." "..player(id, "tiley"))
                            parse("setpos "..id.." 0 0")
                            parse("setpos "..id.." "..x.." "..y)
                        elseif typeID == 57 then
                            parse("setarmor "..id.." 65")

                            parse('sv_soundpos "items/kevlar.wav" '..player(id, "x")..' '..player(id, "y"))
                        elseif typeID == 58 then
                            parse("setarmor "..id.." 100")
                            
                            parse('sv_soundpos "items/kevlar.wav" '..player(id, "x")..' '..player(id, "y"))
                        elseif typeID == 79 or typeID == 80 or typeID == 81 or typeID == 82 or typeID == 83 or typeID == 84 then -- Check if the item is an armor
                            parse("setarmor "..id.." "..(200 + (typeID - 78)))

                            parse('sv_soundpos "items/kevlar.wav" '..player(id, "x")..' '..player(id, "y"))
                        else
                            parse("equip "..id.." "..typeID)

                            if typeID ~= 64 and typeID ~= 65 and typeID ~= 61 and typeID ~= 62 then
                                parse('sv_soundpos "items/pickup.wav" '..player(id, "x")..' '..player(id, "y"))
                            end
                        end

                        parse("setmoney "..id.." "..player(id, "money") - price)
                    else
                        cbm.errorMsg(id, "You already have this or something better")
                    end
                else
                    cbm.errorMsg(id, "You have insufficient funds")
                end
            else
                cbm.errorMsg(id, "You are not in a buyzone")
            end
        else
            cbm.errorMsg(id, "Buytime passed - you can't buy anymore")
        end

        cbm.open(id, content)
    end
end

if not light then
    function cbm.update(id)
        local ply = cbm.player[id]

        if not ply.open then
            return
        end
        
        local content = ply.content
        local hoveredSlot = cbm.getHoveredSlot(id)

        local title = content[hoveredSlot].title and content[hoveredSlot].title or itemtype(content[hoveredSlot][1], "name")
        local img
        local texts = {}

        if content[hoveredSlot].title then
            for k, v in pairs(content[hoveredSlot].content) do
                if v[1] then
                    local color = cbm.hasEnoughMoney(id, v[2]) and "" or "\169255000000"

                    table.insert(texts, {color..itemtype(v[1], "name"), color.."$"..v[2]})
                end
            end 
        else    
            local typeID = content[hoveredSlot][1]
            local price = content[hoveredSlot][2]

            img = cbm.getItemImagePath(typeID)

            local function addStat(name, value)
                table.insert(texts, {name, value})
            end
            
            local color = cbm.hasEnoughMoney(id, price) and "000255000" or "255000000"
            addStat("\169"..color.."Price", "\169"..color.."$"..price)

            if itemtype(typeID, "dmg") > 0 then
                addStat("Damage", itemtype(typeID, "dmg"))
            end
            if itemtype(typeID, "dmg_z1") > 0 then
                addStat("Damage z1", itemtype(typeID, "dmg_z1"))
            end
            if itemtype(typeID, "dmg_z2") > 0 then
                addStat("Damage z2", itemtype(typeID, "dmg_z2"))
            end

            if itemtype(typeID, "ammo") > 0 then
                addStat("Ammo", itemtype(typeID, "ammoin").." | "..itemtype(typeID, "ammo"))
            end

            if itemtype(typeID, "ammo") > 0 then
                addStat("Rate of Fire", math.floor(60000 / (itemtype(typeID, "rate") * 40)).." r/m")
            end

            if itemtype(typeID, "range") > 0 then
                local ranges = {
                    [0] = "Short",
                    [200] = "Medium",
                    [350] = "Far",
                    [800] = "Very Far"
                }
                local function findRange(range)
                    local r

                    for k, v in pairs(ranges) do
                        if range >= k then
                            r = v
                        end               
                    end

                    return r
                end

                addStat("Range", findRange(itemtype(typeID, "range")).." ("..itemtype(typeID, "range")..")")
            end

            if itemtype(typeID, "ammo") > 0 then
                local accuracies = {
                    [0] = "Perfect",
                    [1] = "Good",
                    [2] = "Average",
                    [4] = "Bad",
                    [8] = "Very Bad"
                }
                local function findAccuracy(accuracy)
                    local a

                    for k, v in pairs(accuracies) do
                        if accuracy >= k then
                            a = v
                        end               
                    end

                    return a
                end

                addStat("Dispersion", findAccuracy(itemtype(typeID, "dispersion")))
            end

            if itemtype(typeID, "ammo") > 0 then
                addStat("Recoil", itemtype(typeID, "recoil"))
            end
        end

        cbm.updateSidePanel(id, title, img, texts)
    end
end

function cbm.close(id)
    local ply = cbm.player[id]

    if not ply.open then
        return
    end

    if not cbm.light then
        cbm.removeSidePanel(id)
    end

    freeimage(ply.image)
    ply.image = nil

    for k, v in pairs(ply.lineImage) do
        freeimage(v)
    end
    ply.lineImage = {}

    for i = 110, 128 do
        parse("hudtxt2 "..id.." "..i)
    end

    ply.open = false
    ply.content = {}
end

---------------------
--  HOOKS & BINDS  --
---------------------

addbind("B")
addbind("escape")
addbind("mouse1")

addhook("join", "cbm.join")
function cbm.join(id)
    cbm.player[id] = deepcopy(cbm.playerInit)
end

addhook("menu", "cbm.menu")
function cbm.menu(id, title, button)
    if title == "Circular Buy Menu" then
        if button == 0 then
            cbm.close(id)
        else
            cbm.use(id, button)
        end
    end
end

addhook("key", "cbm.key")
function cbm.key(id, key, state)
    if key == "B" then
        if state == 0 then
            if map("nobuying") == 0 then
                if player(id, "health") > 0 then
                        cbm.open(id, cbm[player(id, "team")])
                end
            else
                cbm.errorMsg(id, "Buying is not allowed")
            end
        end
    elseif key == "escape" then
        if state == 1 then
            cbm.close(id)
        end
    elseif key == "mouse1" then
        if state == 0 then
            cbm.use(id, cbm.getHoveredSlot(id))
        end
    end
end

addhook("leave", "cbm.leave")
function cbm.leave(id)
    cbm.close(id)
end

addhook("die", "cbm.die")
function cbm.die(id)
    cbm.close(id)
end

addhook("startround", "cbm.startround")
function cbm.startround()
    for _, id in pairs(player(0, "tableliving")) do
        cbm.close(id)
    end
end

if not cbm.light then
    addhook("always", "cbm.always")
    function cbm.always()
        for _, id in pairs(player(0, "tableliving")) do
            cbm.update(id)
        end
    end
end